#ifndef _PopulationModels_h
#define _PopulationModels_h





#include "MUQ/Modelling/ModPiece.h"


namespace ecology{

/** This class implements the right hand side of a predator prey model.  The ODE model is:
 *  \f{eqnarray*}{
 *  \frac{dP}{dt} & = & rP\left(1-\frac{P}{K}\right) - s\frac{PQ}{a+P}\\
 *  \frac{dQ}{dt} & = & u\frac{PQ}{a+P} - vQ
 *  \f}
 * The first input to this model is the time, then the current populations: P and Q.  The third input is a six dimensinoal vector holding the remaining model parameters: r, K, s, a, u, and v
*/
class PredPreyRHS : public muq::Modelling::ModPiece {
public:

  PredPreyRHS();
  virtual ~PredPreyRHS() = default;

private:

	// evaluate the right hand side of the predator prey ODE
  virtual Eigen::VectorXd EvaluateImpl(std::vector<Eigen::VectorXd> const& inputs) override;
  virtual Eigen::MatrixXd JacobianImpl(std::vector<Eigen::VectorXd> const& inputs, int inputDimWrt) override;
  
};

/** Combine the PredPreyRHS model with an OdeModPiece to create a ModPiece for the entire forward model. */ 
std::shared_ptr<muq::Modelling::ModPiece> CreatePredPreyModel(double totalTime, int numObservations);
	
	
double EquilibriumPredicate(Eigen::VectorXd const& input);
double StabilityPredicate(Eigen::VectorXd const& input);

}

#endif