#ifndef SimpleTestDensities_h_
#define SimpleTestDensities_h_


#include "MUQ/Modelling/Density.h"
#include "MUQ/Modelling/ModPieceTemplates.h"

#include <boost/iterator/iterator_concepts.hpp>
#include <boost/math/distributions/students_t.hpp>


/** The BOD forward model.  The input is  */
class BODModel : public muq::Modelling::OneInputJacobianModPiece{
  
public:
  BODModel(const Eigen::VectorXd &obsLocsIn) : muq::Modelling::OneInputJacobianModPiece(2,obsLocsIn.size()),
  numEvals(0), numJacEvals(0),
  obsLocs(obsLocsIn){}
  
  
  int GetNumEvals() const{return numEvals;};
  int GetNumGradEvals() const{return numJacEvals;};
private:
  
  virtual Eigen::VectorXd EvaluateImpl(const Eigen::VectorXd &input) override
  {
    numEvals++;
    return input(0)*(1.0-(-1.0*input(1)*obsLocs.array()).exp()).matrix();	
  }
  
  virtual Eigen::MatrixXd JacobianImpl(Eigen::VectorXd const& input) override
  {
    numJacEvals++;
    Eigen::MatrixXd jac(obsLocs.size(),2);
    Eigen::VectorXd expPart = (-1.0*input(1)*obsLocs.array()).exp().matrix();
    jac.col(0) = Eigen::VectorXd::Ones(jac.rows())-expPart;
    jac.col(1) = input(0)*obsLocs.array()*expPart.array();
    
    return jac;	
  }
  
  int numEvals, numJacEvals;
  Eigen::VectorXd obsLocs;
};



#endif
